/**
 * Many thanks to Chris Pederick (http://chrispederick.com/)
 * for his excelent WebDeveloper extension (http://chrispederick.com/work/firefox/webdeveloper/)
 * which is the source of inspiration for this one extension.
 *
 * There is no magic in this extension, it just invokes
 * 'markupDocumentViewer.authorStyleDisabled' property of the browser.
 * The same command is available under 'View' > 'Page Style' menu.
 *
 * TODO: make extension using nsIStyleSheetService.idl
 *       (http://developer.mozilla.org/en/docs/Using_the_Stylesheet_Service)
 */

// Global variables
var gSelectedTab = -1;

// Returns the current content document
function _getContentDocument() {
    return window.top.getBrowser().browsers[window.top.getBrowser().mTabBox.selectedIndex].contentDocument;
}

// Returns the current content window
function _getContentWindow() {
    return window.top.getBrowser().browsers[window.top.getBrowser().mTabBox.selectedIndex].contentWindow;
}

// Gets all the documents from the current page
function _getDocuments(frame) {
    var documents = new Array();

    // If the frame is set
    if (frame) {
        // If the frame document exists
        if (frame.document) {
            documents.push(frame.document);
        }

        for (var i = 0; i < frame.frames.length; i++) {
            documents = documents.concat(_getDocuments(frame.frames[i]));
        }
    }

    return documents;
}

// Initializes the extension
function initialize() {
    // Try to remove the event listener
    try {
        window.removeEventListener("load", callbackPageLoad, true);
    } catch(ex) {
    }

    window.addEventListener("load", callbackPageLoad, true);
}

// Handles the page being loaded
function callbackPageLoad(event) {
    // If the  page is the target
    if (event.target
            && event.target.hasAttribute
            && event.target.hasAttribute("id")
            && event.target.getAttribute("id") == "content") {
        var windowContent = window.document.getElementById("content");

        // If the content exists
        if (windowContent) {
            var tabBox = windowContent.mTabBox;

            // Try to remove the event listener
            try {
                tabBox.removeEventListener("select", callbackTabSelect, true);
            } catch(ex) {
            }

            tabBox.addEventListener("select", callbackTabSelect, true);

            updateCommandStates();
        }
    }
}

// Handles a tab being selected
function callbackTabSelect(event) {
    var selectedTab = getBrowser().mTabBox.selectedIndex;

    // If the selected tab is different
    if (selectedTab != gSelectedTab) {
        gSelectedTab = selectedTab;
    } else {
        // This time the event handler is called the second time
        updateCommandStates();
    }
}

function updateCommandStates() {
    try {
        var markupDocumentViewer = getBrowser().markupDocumentViewer;
        document.getElementById('cmd_disableStyles').setAttribute('checked',
                markupDocumentViewer.authorStyleDisabled);
    } catch(ex) {
    }

    var doc = _getContentDocument();

    document.getElementById('cmd_disableColors').setAttribute('checked',
                doc._colorsDisabled == true);

    document.getElementById('cmd_disableFonts').setAttribute('checked',
                doc._fontsDisabled == true);
}

function enableStyles() {
    try {
        var markupDocumentViewer = getBrowser().markupDocumentViewer;
        markupDocumentViewer.authorStyleDisabled = false;
    } catch(ex) {
    }
}

function disableStyles() {
    try {
        var markupDocumentViewer = getBrowser().markupDocumentViewer;
        markupDocumentViewer.authorStyleDisabled = true;
    } catch(ex) {
    }
}

function enableColors() {
    var doc = _getContentDocument();
    removeStyleSheet("disable-colors");
    doc._colorsDisabled = false;
}

function enableFonts() {
    var doc = _getContentDocument();
    removeStyleSheet("disable-fonts");
    doc._fontsDisabled = false;
}

function addStyleSheet(location, id) {
    var documents = _getDocuments(_getContentWindow());
    for (var i = 0; i < documents.length; i++) {
        var pageDocument = documents[i];
        var headElements = pageDocument.getElementsByTagName("head");
        var linkElement = pageDocument.createElement("link");

        linkElement.setAttribute("href", location);
        linkElement.setAttribute("id", id);
        linkElement.setAttribute("rel", "stylesheet");
        linkElement.setAttribute("type", "text/css");

        // If there is a head element
        if (headElements.length > 0) {
            headElements[0].appendChild(linkElement);
        } else {
            pageDocument.documentElement.appendChild(linkElement);
        }
    }
}

// Removes the style sheet with the given id
function removeStyleSheet(id) {
    var documents = _getDocuments(_getContentWindow());
    for (var i = 0; i < documents.length; i++) {
        var element = documents[i].getElementById(id);

        // If the element is set
        if (element) {
            element.parentNode.removeChild(element);
        }
    }
}

function toggleStyles() {
    try {
        var markupDocumentViewer = getBrowser().markupDocumentViewer;
        if (markupDocumentViewer.authorStyleDisabled) {
            markupDocumentViewer.authorStyleDisabled = false;
        } else {
            enableColors();
            enableFonts();
            markupDocumentViewer.authorStyleDisabled = true;
        }
    } catch(ex) {
    }
    updateCommandStates();
}

function toggleColors() {
    var doc = _getContentDocument();
    if (doc._colorsDisabled == true) {
        removeStyleSheet("disable-colors");
        doc._colorsDisabled = false;
    } else {
        enableStyles();
        addStyleSheet("chrome://readeasily/content/stylesheet/disable-colors.css", "disable-colors");
        doc._colorsDisabled = true;
    }
    updateCommandStates();
}

function toggleFonts() {
    var doc = _getContentDocument();
    if (doc._fontsDisabled == true) {
        removeStyleSheet("disable-fonts");
        doc._fontsDisabled = false;
    } else {
        enableStyles();
        addStyleSheet("chrome://readeasily/content/stylesheet/disable-fonts.css", "disable-fonts");
        doc._fontsDisabled = true;
    }
    updateCommandStates();
}

// Initialize extension
initialize();